<?php
/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author         Varun Shoor
 *
 * @package        SWIFT
 * @copyright      Copyright (c) 2001-2012, Kayako
 * @license        http://www.kayako.com/license
 * @link           http://www.kayako.com
 *
 * ###############################################
 */

require_once('modules/ky_Tickets/KayakoAPILibrary/kyIncludes.php');

class KayakoConnection
{
	/**
	 * email of requester
	 *
	 * @var string
	 */
	public $email = null;

	/**
	 * Initialize client configuration object.
	 *
	 * @params $apiUrl, $apiKey, $secretKey
	 *
	 * @author Kayako
	 */
	public function KayakoConnection($apiUrl, $apiKey, $secretKey)
	{
		$config = new kyConfig($apiUrl, $apiKey, $secretKey);
		$config->setIsStandardURLType(true);
		kyConfig::set($config);
	}

	/**
	 * Fetch user tickets
	 *
	 * @return kyTicket object on success, else fail
	 *
	 * @author Kayako
	 */
	public function view()
	{
		$department = kyDepartment::getAll();
		$user       = kyUser::search($this->email);

		if (sizeof($user->getRawArray()) > 0) {
			$this->tickets = kyTicket::getAll($department, array(), array(), $user)->getRawArray();
		} else {
			$this->tickets = null;
		}
	}

	/**
	 * Fetch all departments
	 *
	 * @return bool true on success, else false
	 *
	 * @author Kayako
	 */
	public function fetchAllDepartment()
	{
		$options_array = array();

		$departments_tree = $this->getDepartmentsTree();
		foreach ($departments_tree as $key => $val) {
			$options_array[$key] = $val['department'];
			foreach ($val['child_departments'] as $child_dept_key => $child_dept_value) {
				$options_array[$child_dept_key] = $child_dept_value;
			}
		}
		$this->departments = $options_array;
		$this->priority    = kyTicketPriority::getAll()->getRawArray();
		$this->status      = kyTicketStatus::getAll();
		$this->type        = kyTicketType::getAll();
		$this->owner       = kyStaff::getAll(array())->getRawArray();

		return true;
	}

	/**
	 * Create a new ticket
	 *
	 * @param array $ticketArray Post data while converting leads into ticket
	 *
	 * @return bool true on success, else false
	 *
	 * @author Kayako
	 */
	public function postTicket($ticketArray)
	{
		kyTicket::setDefaults($ticketArray['status'], $ticketArray['priority'], $ticketArray['type']);

		$staff    = kyStaff::getAll(array());
		$staffObj = $this->searchStaff($staff->getRawArray(), $ticketArray['kemail']);

		$ownerObj = $ticketArray['owner'] != 0 ? kyStaff::get($ticketArray['owner']) : null;

		if (is_object($staffObj)) {
			$departmentObj  = kyDepartment::get($ticketArray['department']);
			$ticketObj      = $staffObj->newTicket($departmentObj, $ticketArray['content'], $ticketArray['subject'])->setEmail($ticketArray['lead_mail'])->create();

			if ($ownerObj != null)
			{
				$ticketObj->setOwnerStaff($ownerObj)->update();
			}

			$ticket_mask_id = $ticketObj->getDisplayId();

			return $ticket_mask_id;
		} else {
			return false;
		}
	}


	/**
	 * Sort departments by parent id
	 *
	 * @return object kyDepartment
	 *
	 * @author Kayako
	 */
	protected function getDepartmentsTree()
	{
		$departments_tree = array();
		$all_departments  = kyDepartment::getAll()->filterByModule(kyDepartment::MODULE_TICKETS)->filterByType(kyDepartment::TYPE_PUBLIC);
		$top_departments  = $all_departments->filterByParentDepartmentId(null)->orderByDisplayOrder();
		foreach ($top_departments as $top_department) {
			$departments_tree[$top_department->getId()]                      = array(
				'department' => $top_department->getTitle()
			);
			$departments_tree[$top_department->getId()]['child_departments'] = array();
			foreach ($all_departments->filterByParentDepartmentId($top_department->getId())->orderByDisplayOrder() as $child_department) {
				$departments_tree[$top_department->getId()]['child_departments'][$child_department->getId()] = $child_department->getTitle();
			}
		}

		return $departments_tree;
	}

	/**
	 * Sort departments by parent id
	 *
	 * @param  array $staff_array Staff List
	 * @param string $email email of staff to be searched
	 *
	 * @return object kyStaff
	 *
	 * @author Kayako
	 */
	protected function searchStaff($staff_array, $email = "")
	{
		$staffObj = null;
		foreach ($staff_array as $staff) {
			if ($staff->getEmail() === $email) {
				$staffObj = $staff;
				break;
			}
		}

		return $staffObj;
	}
}